bash python/jittor/test/system/test_cuda10.0_ubuntu16.04.sh
bash python/jittor/test/system/test_cuda10.0_ubuntu18.04.sh
bash python/jittor/test/system/test_cuda11.1_ubuntu16.04.sh
bash python/jittor/test/system/test_cuda11.1_ubuntu18.04.sh
bash python/jittor/test/system/test_cuda11.1_ubuntu20.04.sh
bash python/jittor/test/system/test_nocuda_ubuntu18.04.sh
